@@ -23,10 +23,12 @@
 
 #include <gst/gst.h>
 #include <tremor/ivorbiscodec.h>
+#include "vorbisdec.h"
+
+GST_DEBUG_CATEGORY (vorbisdec_debug);
 
 extern GType ivorbisfile_get_type (void);
 
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -34,6 +36,13 @@ plugin_init (GstPlugin * plugin)
           ivorbisfile_get_type ()))
     return FALSE;
 
+  if (!gst_element_register (plugin, "ivorbisdec", GST_RANK_SECONDARY,
+          gst_ivorbis_dec_get_type ()))
+    return FALSE;
+
+  GST_DEBUG_CATEGORY_INIT (vorbisdec_debug, "ivorbisdec", 0,
+      "vorbis decoding element (integer decoder)");
+
   return TRUE;
 }
 
