@@ -206,7 +206,6 @@ struct _GstBaseParsePrivate
   gboolean flushing;
 
   gint64 offset;
-  gint64 pending_offset;
 
   GList *pending_events;
 
@@ -291,7 +290,7 @@ gst_base_parse_base_init (gpointer g_class)
   GstBaseParseClass *klass = GST_BASE_PARSE_CLASS (g_class);
   GstBaseParseClassPrivate *priv;
 
-  GST_DEBUG_CATEGORY_INIT (gst_base_parse_debug, "flacbaseparse", 0,
+  GST_DEBUG_CATEGORY_INIT (gst_base_parse_debug, "amrbaseparse", 0,
       "baseparse element");
 
   /* TODO: Remove this once GObject supports class private data */
@@ -547,7 +546,7 @@ gst_base_parse_sink_eventfunc (GstBaseParse * parse, GstEvent * event)
     {
       gdouble rate, applied_rate;
       GstFormat format;
-      gint64 start, stop, pos;
+      gint64 start, stop, pos, offset = 0;
       gboolean update;
 
       gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
@@ -559,7 +558,7 @@ gst_base_parse_sink_eventfunc (GstBaseParse * parse, GstEvent * event)
 
         /* stop time is allowed to be open-ended, but not start & pos */
         seg_stop = GST_CLOCK_TIME_NONE;
-        parse->priv->pending_offset = pos;
+        offset = pos;
 
         if (gst_base_parse_bytepos_to_time (parse, start, &seg_start) &&
             gst_base_parse_bytepos_to_time (parse, pos, &seg_pos)) {
@@ -600,15 +599,22 @@ gst_base_parse_sink_eventfunc (GstBaseParse * parse, GstEvent * event)
       gst_event_replace (eventp, event);
       gst_event_unref (event);
       handled = TRUE;
+
+      /* but finish the current segment */
+      GST_DEBUG_OBJECT (parse, "draining current segment");
+      gst_base_parse_drain (parse);
+      gst_adapter_clear (parse->adapter);
+      parse->priv->offset = offset;
       break;
     }
 
     case GST_EVENT_FLUSH_START:
       parse->priv->flushing = TRUE;
+      handled = gst_pad_push_event (parse->srcpad, event);
       /* Wait for _chain() to exit by taking the srcpad STREAM_LOCK */
       GST_PAD_STREAM_LOCK (parse->srcpad);
-      handled = gst_pad_push_event (parse->srcpad, event);
       GST_PAD_STREAM_UNLOCK (parse->srcpad);
+
       break;
 
     case GST_EVENT_FLUSH_STOP:
@@ -729,7 +735,6 @@ gst_base_parse_handle_and_push_buffer (GstBaseParse * parse,
     GstBaseParseClass * klass, GstBuffer * buffer)
 {
   GstFlowReturn ret;
-  GstClockTime last_stop = GST_CLOCK_TIME_NONE;
 
   if (parse->priv->discont) {
     GST_DEBUG_OBJECT (parse, "marking DISCONT");
@@ -743,48 +748,113 @@ gst_base_parse_handle_and_push_buffer (GstBaseParse * parse,
    *        keep track of timestamp and calculate everything possible
    *        if not set already */
 
+  /* First buffers are dropped, this means that the subclass needs more
+   * frames to decide on the format and queues them internally */
+  if (ret == GST_BASE_PARSE_FLOW_DROPPED && !GST_PAD_CAPS (parse->srcpad)) {
+    gst_buffer_unref (buffer);
+    return GST_FLOW_OK;
+  }
+
+  /* convert internal flow to OK and mark discont for the next buffer. */
+  if (ret == GST_BASE_PARSE_FLOW_DROPPED) {
+    parse->priv->discont = TRUE;
+    ret = GST_FLOW_OK;
+
+    gst_buffer_unref (buffer);
+
+    return ret;
+  } else if (ret != GST_FLOW_OK) {
+    return ret;
+  }
+
+  return gst_base_parse_push_buffer (parse, buffer);
+}
+
+/**
+ * gst_base_parse_push_buffer:
+ * @parse: #GstBaseParse.
+ * @buffer: #GstBuffer.
+ *
+ * Pushes the buffer downstream, sends any pending events and
+ * does some timestamp and segment handling.
+ *
+ * This must be called with srcpad STREAM_LOCK held.
+ *
+ * Returns: #GstFlowReturn
+ */
+GstFlowReturn
+gst_base_parse_push_buffer (GstBaseParse * parse, GstBuffer * buffer)
+{
+  GstFlowReturn ret = GST_FLOW_OK;
+  GstClockTime last_stop = GST_CLOCK_TIME_NONE;
+
   if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer))
     last_stop = GST_BUFFER_TIMESTAMP (buffer);
   if (last_stop != GST_CLOCK_TIME_NONE && GST_BUFFER_DURATION_IS_VALID (buffer))
     last_stop += GST_BUFFER_DURATION (buffer);
 
+  /* should have caps by now */
+  g_return_val_if_fail (GST_PAD_CAPS (parse->srcpad), GST_FLOW_ERROR);
+
   gst_buffer_set_caps (buffer, GST_PAD_CAPS (parse->srcpad));
 
+  /* and should then also be linked downstream, so safe to send some events */
+  if (parse->priv->pad_mode == GST_ACTIVATE_PULL) {
+    if (G_UNLIKELY (parse->close_segment)) {
+      GST_DEBUG_OBJECT (parse, "loop sending close segment");
+      gst_pad_push_event (parse->srcpad, parse->close_segment);
+      parse->close_segment = NULL;
+    }
+
+    if (G_UNLIKELY (parse->pending_segment)) {
+      GST_DEBUG_OBJECT (parse, "loop push pending segment");
+      gst_pad_push_event (parse->srcpad, parse->pending_segment);
+      parse->pending_segment = NULL;
+    }
+  } else {
+    if (G_UNLIKELY (parse->pending_segment)) {
+      GST_DEBUG_OBJECT (parse, "chain pushing a pending segment");
+      gst_pad_push_event (parse->srcpad, parse->pending_segment);
+      parse->pending_segment = NULL;
+    }
+  }
+
+  if (G_UNLIKELY (parse->priv->pending_events)) {
+    GList *l;
+
+    for (l = parse->priv->pending_events; l != NULL; l = l->next) {
+      gst_pad_push_event (parse->srcpad, GST_EVENT (l->data));
+    }
+    g_list_free (parse->priv->pending_events);
+    parse->priv->pending_events = NULL;
+  }
+
   /* TODO: Add to seek table */
 
-  if (ret == GST_FLOW_OK) {
-    if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer) &&
-        GST_CLOCK_TIME_IS_VALID (parse->segment.stop) &&
-        GST_BUFFER_TIMESTAMP (buffer) > parse->segment.stop) {
-      GST_LOG_OBJECT (parse, "Dropped frame, after segment");
-      gst_buffer_unref (buffer);
-    } else if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer) &&
-        GST_BUFFER_DURATION_IS_VALID (buffer) &&
-        GST_CLOCK_TIME_IS_VALID (parse->segment.start) &&
-        GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION_IS_VALID (buffer)
-        < parse->segment.start) {
-      /* FIXME: subclass needs way to override the start as downstream might
-       * need frames before for proper decoding */
-      GST_LOG_OBJECT (parse, "Dropped frame, before segment");
-      gst_buffer_unref (buffer);
-    } else {
-      ret = gst_pad_push (parse->srcpad, buffer);
-      GST_LOG_OBJECT (parse, "frame (%d bytes) pushed: %d",
-          GST_BUFFER_SIZE (buffer), ret);
-    }
-  } else {
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer) &&
+      GST_CLOCK_TIME_IS_VALID (parse->segment.stop) &&
+      GST_BUFFER_TIMESTAMP (buffer) > parse->segment.stop) {
+    GST_LOG_OBJECT (parse, "Dropped frame, after segment");
     gst_buffer_unref (buffer);
+  } else if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer) &&
+      GST_BUFFER_DURATION_IS_VALID (buffer) &&
+      GST_CLOCK_TIME_IS_VALID (parse->segment.start) &&
+      GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer)
+      < parse->segment.start) {
+    /* FIXME: subclass needs way to override the start as downstream might
+     * need frames before for proper decoding */
+    GST_LOG_OBJECT (parse, "Dropped frame, before segment");
+    gst_buffer_unref (buffer);
+  } else {
+    ret = gst_pad_push (parse->srcpad, buffer);
+    GST_LOG_OBJECT (parse, "frame (%d bytes) pushed: %d",
+        GST_BUFFER_SIZE (buffer), ret);
   }
 
   /* Update current running segment position */
   if (ret == GST_FLOW_OK && last_stop != GST_CLOCK_TIME_NONE)
     gst_segment_set_last_stop (&parse->segment, GST_FORMAT_TIME, last_stop);
 
-  /* convert internal flow to OK and mark discont for the next buffer. */
-  if (ret == GST_BASE_PARSE_FLOW_DROPPED) {
-    parse->priv->discont = TRUE;
-    ret = GST_FLOW_OK;
-  }
   return ret;
 }
 
@@ -802,6 +872,8 @@ gst_base_parse_drain (GstBaseParse * parse)
 {
   guint avail;
 
+  GST_DEBUG_OBJECT (parse, "draining");
+
   for (;;) {
     avail = gst_adapter_available (parse->adapter);
     if (!avail)
@@ -811,6 +883,12 @@ gst_base_parse_drain (GstBaseParse * parse)
     if (gst_base_parse_chain (parse->sinkpad, NULL) != GST_FLOW_OK) {
       break;
     }
+
+    /* nothing changed, maybe due to truncated frame; break infinite loop */
+    if (avail == gst_adapter_available (parse->adapter)) {
+      GST_DEBUG_OBJECT (parse, "no change during draining; flushing");
+      gst_adapter_clear (parse->adapter);
+    }
   }
 }
 
@@ -838,32 +916,6 @@ gst_base_parse_chain (GstPad * pad, GstBuffer * buffer)
   parse = GST_BASE_PARSE (GST_OBJECT_PARENT (pad));
   bclass = GST_BASE_PARSE_GET_CLASS (parse);
 
-  if (G_UNLIKELY (parse->pending_segment)) {
-    GST_DEBUG_OBJECT (parse, "chain pushing a pending segment");
-    gst_pad_push_event (parse->srcpad, parse->pending_segment);
-    parse->pending_segment = NULL;
-    parse->priv->offset = parse->priv->pending_offset;
-
-    /* Make sure that adapter doesn't have any old data after
-       newsegment has been pushed */
-
-    /* FIXME: when non-flushing seek occurs, chain is still processing the
-       data from old segment. If this processing loop is then interrupted
-       (e.g. paused), chain function exists and next time it gets called
-       all this old data gets lost and playback continues from new segment */
-    gst_adapter_clear (parse->adapter);
-  }
-
-  if (G_UNLIKELY (parse->priv->pending_events)) {
-    GList *l;
-
-    for (l = parse->priv->pending_events; l != NULL; l = l->next) {
-      gst_pad_push_event (parse->srcpad, GST_EVENT (l->data));
-    }
-    g_list_free (parse->priv->pending_events);
-    parse->priv->pending_events = NULL;
-  }
-
   if (G_LIKELY (buffer)) {
     GST_LOG_OBJECT (parse, "buffer size: %d, offset = %lld",
         GST_BUFFER_SIZE (buffer), GST_BUFFER_OFFSET (buffer));
@@ -875,12 +927,12 @@ gst_base_parse_chain (GstPad * pad, GstBuffer * buffer)
   while (!parse->priv->flushing) {
     tmpbuf = gst_buffer_new ();
 
-    GST_BASE_PARSE_LOCK (parse);
-    min_size = parse->priv->min_frame_size;
-    GST_BASE_PARSE_UNLOCK (parse);
-
     /* Synchronization loop */
     for (;;) {
+      GST_BASE_PARSE_LOCK (parse);
+      min_size = parse->priv->min_frame_size;
+      GST_BASE_PARSE_UNLOCK (parse);
+
       /* Collect at least min_frame_size bytes */
       if (gst_adapter_available (parse->adapter) < min_size) {
         GST_DEBUG_OBJECT (parse, "not enough data available (only %d bytes)",
@@ -902,6 +954,13 @@ gst_base_parse_chain (GstPad * pad, GstBuffer * buffer)
 
       skip = -1;
       if (bclass->check_valid_frame (parse, tmpbuf, &fsize, &skip)) {
+        if (gst_adapter_available (parse->adapter) < fsize) {
+          GST_DEBUG_OBJECT (parse,
+              "found valid frame but not enough data available (only %d bytes)",
+              gst_adapter_available (parse->adapter));
+          gst_buffer_unref (tmpbuf);
+          goto done;
+        }
         break;
       }
       if (skip > 0) {
@@ -1051,18 +1110,6 @@ gst_base_parse_loop (GstPad * pad)
   parse = GST_BASE_PARSE (gst_pad_get_parent (pad));
   klass = GST_BASE_PARSE_GET_CLASS (parse);
 
-  if (parse->close_segment) {
-    GST_DEBUG_OBJECT (parse, "loop sending close segment");
-    gst_pad_push_event (parse->srcpad, parse->close_segment);
-    parse->close_segment = NULL;
-  }
-
-  if (parse->pending_segment) {
-    GST_DEBUG_OBJECT (parse, "loop push pending segment");
-    gst_pad_push_event (parse->srcpad, parse->pending_segment);
-    parse->pending_segment = NULL;
-  }
-
   /* TODO: Check if we reach segment stop limit */
 
   while (TRUE) {
@@ -1567,12 +1614,13 @@ gst_base_parse_handle_seek (GstBaseParse * parse, GstEvent * event)
    * it directly or fail. For TIME, try upstream, but do it ourselves if
    * it fails upstream */
   if (format != GST_FORMAT_TIME) {
-    gst_event_ref (event);
     return gst_pad_push_event (parse->sinkpad, event);
   } else {
     gst_event_ref (event);
-    if (gst_pad_push_event (parse->sinkpad, event))
+    if (gst_pad_push_event (parse->sinkpad, event)) {
+      gst_event_unref (event);
       return TRUE;
+    }
   }
 
   /* get flush flag */
@@ -1722,18 +1770,14 @@ gst_base_parse_sink_setcaps (GstPad * pad, GstCaps * caps)
   GstBaseParseClass *klass;
   gboolean res = TRUE;
 
-  gchar *caps_str = gst_caps_to_string (caps);
-  g_free (caps_str);
-
-  parse = GST_BASE_PARSE (gst_pad_get_parent (pad));
+  parse = GST_BASE_PARSE (GST_PAD_PARENT (pad));
   klass = GST_BASE_PARSE_GET_CLASS (parse);
 
-  GST_DEBUG_OBJECT (parse, "setcaps: %s", caps_str);
+  GST_DEBUG_OBJECT (parse, "caps: %" GST_PTR_FORMAT, caps);
 
   if (klass->set_sink_caps)
     res = klass->set_sink_caps (parse, caps);
 
   parse->negotiated = res;
-  gst_object_unref (parse);
-  return gst_pad_set_caps (pad, caps);
+  return res && gst_pad_set_caps (pad, caps);
 }
