@@ -241,7 +241,8 @@ mpegts_packetizer_parse_section_header (MpegTSPacketizer * packetizer,
   data = GST_BUFFER_DATA (section->buffer);
 
   section->table_id = *data++;
-  if ((data[0] & 0x80) == 0)
+  /* if table_id is 0 (pat) then ignore the subtable extension */
+  if ((data[0] & 0x80) == 0 || section->table_id == 0)
     section->subtable_extension = 0;
   else
     section->subtable_extension = GST_READ_UINT16_BE (data + 2);
@@ -502,6 +503,55 @@ mpegts_packetizer_parse_pmt (MpegTSPacketizer * packetizer,
     g_free (struct_name);
 
     if (stream_info_length) {
+      /* check for AC3 descriptor */
+      GstMPEGDescriptor *desc =
+          gst_mpeg_descriptor_parse (data, stream_info_length);
+      if (desc != NULL) {
+        guint8 *desc_data;
+        if (gst_mpeg_descriptor_find (desc, DESC_DVB_AC3)) {
+          gst_structure_set (stream_info, "has-ac3", G_TYPE_BOOLEAN, TRUE,
+              NULL);
+        }
+        desc_data = gst_mpeg_descriptor_find (desc, DESC_DVB_DATA_BROADCAST_ID);
+        if (desc_data) {
+          guint16 data_broadcast_id;
+          data_broadcast_id =
+              DESC_DVB_DATA_BROADCAST_ID_data_broadcast_id (desc_data);
+          gst_structure_set (stream_info, "data-broadcast-id", G_TYPE_UINT,
+              data_broadcast_id, NULL);
+        }
+        desc_data = gst_mpeg_descriptor_find (desc, DESC_DVB_DATA_BROADCAST);
+        if (desc_data) {
+          GstStructure *databroadcast_info;
+          guint16 data_broadcast_id;
+          guint8 component_tag;
+          data_broadcast_id =
+              DESC_DVB_DATA_BROADCAST_data_broadcast_id (desc_data);
+          component_tag = DESC_DVB_DATA_BROADCAST_component_tag (desc_data);
+          databroadcast_info = gst_structure_new ("data-broadcast", "id",
+              G_TYPE_UINT, data_broadcast_id, "component-tag", component_tag,
+              NULL);
+          gst_structure_set (stream_info, "data-broadcast", GST_TYPE_STRUCTURE,
+              databroadcast_info, NULL);
+        }
+        desc_data =
+            gst_mpeg_descriptor_find (desc, DESC_DVB_CAROUSEL_IDENTIFIER);
+        if (desc_data) {
+          guint32 carousel_id;
+          carousel_id = DESC_DVB_CAROUSEL_IDENTIFIER_carousel_id (desc_data);
+          gst_structure_set (stream_info, "carousel-id", G_TYPE_UINT,
+              carousel_id, NULL);
+        }
+        desc_data = gst_mpeg_descriptor_find (desc, DESC_DVB_STREAM_IDENTIFIER);
+        if (desc_data) {
+          guint8 component_tag;
+          component_tag = DESC_DVB_STREAM_IDENTIFIER_component_tag (desc_data);
+          gst_structure_set (stream_info, "component-tag", G_TYPE_UINT,
+              component_tag, NULL);
+        }
+        gst_mpeg_descriptor_free (desc);
+      }
+
       descriptors = g_value_array_new (0);
       if (!mpegts_packetizer_parse_descriptors (packetizer,
               &data, data + stream_info_length, descriptors)) {
@@ -514,6 +564,7 @@ mpegts_packetizer_parse_pmt (MpegTSPacketizer * packetizer,
       gst_structure_set (stream_info,
           "descriptors", G_TYPE_VALUE_ARRAY, descriptors, NULL);
       g_value_array_free (descriptors);
+
     }
 
     g_value_init (&stream_value, GST_TYPE_STRUCTURE);
@@ -1056,12 +1107,12 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
         GValue frequencies = { 0 };
         guint8 type;
 
-        g_value_init (&frequencies, GST_TYPE_LIST);
         type = *current_pos & 0x03;
         current_pos++;
 
         if (type) {
           const gchar *fieldname = NULL;
+          g_value_init (&frequencies, GST_TYPE_LIST);
 
           while (current_pos < delivery + DESC_LENGTH (delivery) - 3) {
             guint32 freq = 0;
@@ -1251,17 +1302,16 @@ mpegts_packetizer_parse_sdt (MpegTSPacketizer * packetizer,
     service_id = GST_READ_UINT16_BE (data);
     data += 2;
 
-    /* reserved */
+    EIT_schedule = ((*data & 0x02) == 2);
+    EIT_present_following = (*data & 0x01) == 1;
+
     data += 1;
-
     tmp = GST_READ_UINT16_BE (data);
-    data += 2;
 
-    EIT_schedule = (tmp >> 15);
-    EIT_present_following = (tmp >> 14) & 0x01;
-    running_status = (tmp >> 5) & 0x03;
-    scrambled = (tmp >> 4) & 0x01;
+    running_status = (*data >> 5) & 0x07;
+    scrambled = (*data >> 4) & 0x01;
     descriptors_loop_length = tmp & 0x0FFF;
+    data += 2;
 
     /* TODO send tag event down relevant pad for channel name and provider */
     service_name = g_strdup_printf ("service-%d", service_id);
@@ -1294,7 +1344,26 @@ mpegts_packetizer_parse_sdt (MpegTSPacketizer * packetizer,
             (gchar *) DESC_DVB_SERVICE_name_text (service_descriptor);
         if (servicename_length + serviceprovider_name_length + 2 <=
             DESC_LENGTH (service_descriptor)) {
-
+          gchar *running_status_tmp;
+          switch (running_status) {
+            case 0:
+              running_status_tmp = "undefined";
+              break;
+            case 1:
+              running_status_tmp = "not running";
+              break;
+            case 2:
+              running_status_tmp = "starts in a few seconds";
+              break;
+            case 3:
+              running_status_tmp = "pausing";
+              break;
+            case 4:
+              running_status_tmp = "running";
+              break;
+            default:
+              running_status_tmp = "reserved";
+          }
           servicename_tmp =
               get_encoding_and_convert (servicename, servicename_length);
           serviceprovider_name_tmp =
@@ -1304,7 +1373,8 @@ mpegts_packetizer_parse_sdt (MpegTSPacketizer * packetizer,
           gst_structure_set (service,
               "name", G_TYPE_STRING, servicename_tmp,
               "provider-name", G_TYPE_STRING, serviceprovider_name_tmp,
-              "scrambled", G_TYPE_BOOLEAN, scrambled, NULL);
+              "scrambled", G_TYPE_BOOLEAN, scrambled,
+              "running-status", G_TYPE_STRING, running_status_tmp, NULL);
 
           g_free (servicename_tmp);
           g_free (serviceprovider_name_tmp);
@@ -1679,7 +1749,9 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
                   break;
               }
               component = gst_structure_new ("video", "high-definition",
-                  G_TYPE_BOOLEAN, TRUE, "frequency", G_TYPE_INT, freq, NULL);
+                  G_TYPE_BOOLEAN, TRUE, "frequency", G_TYPE_INT, freq,
+                  "tag", G_TYPE_INT, DESC_DVB_COMPONENT_tag (comp_descriptor),
+                  NULL);
               if (widescreen == 0) {
                 gst_structure_set (component, "aspect-ratio",
                     G_TYPE_STRING, "4:3", NULL);
@@ -1717,7 +1789,8 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
                   break;
               }
               component = gst_structure_new ("audio", "type", G_TYPE_STRING,
-                  comptype, NULL);
+                  comptype, "tag", G_TYPE_INT,
+                  DESC_DVB_COMPONENT_tag (comp_descriptor), NULL);
               break;
             case 0x03:         /* subtitles/teletext/vbi */
               comptype = "reserved";
@@ -1757,7 +1830,8 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
                   break;
               }
               component = gst_structure_new ("teletext", "type", G_TYPE_STRING,
-                  comptype, NULL);
+                  comptype, "tag", G_TYPE_INT,
+                  DESC_DVB_COMPONENT_tag (comp_descriptor), NULL);
               break;
           }
           if (component) {
@@ -1832,6 +1906,19 @@ mpegts_packetizer_clear (MpegTSPacketizer * packetizer)
   gst_adapter_clear (packetizer->adapter);
 }
 
+void
+mpegts_packetizer_remove_stream (MpegTSPacketizer * packetizer, gint16 pid)
+{
+  MpegTSPacketizerStream *stream =
+      (MpegTSPacketizerStream *) g_hash_table_lookup (packetizer->streams,
+      GINT_TO_POINTER ((gint) pid));
+  if (stream) {
+    GST_INFO ("Removing stream for PID %d", pid);
+
+    g_hash_table_remove (packetizer->streams, GINT_TO_POINTER ((gint) pid));
+  }
+}
+
 MpegTSPacketizer *
 mpegts_packetizer_new ()
 {
