@@ -514,7 +514,11 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
       pid_string = g_value_dup_string (value);
       if (!strcmp (pid_string, "8192")) {
         /* get the whole ts */
+        int pid_count = 1;
         object->pids[0] = 8192;
+        while (pid_count < MAX_FILTERS) {
+          object->pids[pid_count++] = G_MAXUINT16;
+        }
       } else {
         int pid = 0;
         int pid_count;
@@ -539,6 +543,9 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
           }
           pids++;
         }
+        while (pid_count < MAX_FILTERS) {
+          object->pids[pid_count++] = G_MAXUINT16;
+        }
 
         g_strfreev (tmp);
       }
@@ -1319,13 +1326,15 @@ gst_dvbsrc_tune (GstDvbSrc * object)
 #endif
       g_warning ("Error tuning channel: %s", strerror (errno));
     }
-    for (i = 0; i < 5; i++) {
+    for (i = 0; i < 50; i++) {
       usleep (100000);
       if (ioctl (object->fd_frontend, FE_READ_STATUS, &status) == -1) {
         perror ("FE_READ_STATUS");
         break;
       }
       GST_LOG_OBJECT (object, "status == 0x%02x", status);
+      if (status & FE_HAS_LOCK)
+        break;
     }
     if (status & FE_HAS_LOCK)
       break;
